/*$6 ***************************************************************************
Copyright (C) <2007> <Sven Trenkel, Rene Weller, Gabriel Zachmann>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
 ******************************************************************************* */
/**
 *  \brief Methods for configuration computation:
 *         Grid-, sphere- and csphere-method
 *
 *  \author Sven Trenkel, Rene Weller, Gabriel Zachmann
 */
 
#include "PosFinder.h"
#include "Settings.h"
#include "Field.h"


/**
 *  Grid: Implements the grid method described in http://cg.in.tu-clausthal.de/papers/wscg07/wscg_benchmarking_suite.pdf
 */
class Grid : public PosFinder
{

public:
    Grid( Settings *set );
    ~       Grid();
    bool    iterate();
    bool    rotate();

private:
    int     x, y, z, State;
    void    updatePosition( int x, int y, int z, double dist );
    void    makeGrid( int maxsize[3] );
    void    deleteGrid();
    void    test( int x, int y, int z );
    int     size[3];
    double  fieldSize[3];
    Field   ***field;
};


/**
 *  Sphere: Implements the sphere method described in http://cg.in.tu-clausthal.de/papers/wscg07/wscg_benchmarking_suite.pdf
 */
class Sphere : public PosFinder
{

private:
    double  xzDeg, yzDeg;

public:
    bool    iterate();
    bool    rotate();
    Sphere( Settings *set );
};


/**
 *  CheckSphere: Implements an extended version of the sphere method described in http://cg.in.tu-clausthal.de/papers/wscg07/wscg_benchmarking_suite.pdf
 */
class CheckSphere : public PosFinder
{

private:
    double  xzDeg, yzDeg, avDist;

public:
    bool    iterate();
    bool    rotate();
    CheckSphere( Settings *set );
};
